@@ -16,9 +16,17 @@ def runner_argparser():
     parser.add_argument('--runner', default='processes', choices=['processes', 'slurm', 'ngc'], help='Runner backend, use OS multiprocessing by default')
     parser.add_argument('--pause_between', default=10, type=int, help='Pause in seconds between processes')
     parser.add_argument('--num_gpus', default=1, type=int, help='How many GPUs to use')
-    parser.add_argument('--experiments_per_gpu', default=-1, type=int, help='How many experiments can we squeeze on a single GPU (-1 for not altering CUDA_VISIBLE_DEVICES at all)')
     parser.add_argument('--max_parallel', default=4, type=int, help='Maximum simultaneous experiments')
     parser.add_argument('--experiment_suffix', default='', type=str, help='Append this to the name of the experiment dir')
+    parser.add_argument(
+        '--experiments_per_gpu', default=-1, type=int,
+        help='How many experiments can we squeeze on a single GPU. '
+             'Specify this option if and only if you are using runner to run several experiments using OS-level'
+             'parallelism (--runner=processes).'
+             'In any other case use default value (-1) for not altering CUDA_VISIBLE_DEVICES at all.'
+             'This will allow your experiments to use all GPUs available (as many as --num_gpu allows)'
+             'Helpful when e.g. you are running a single big PBT experiment.',
+    )
 
     parser = add_slurm_args(parser)
     parser = add_ngc_args(parser)
